@@ -59,35 +59,38 @@ module YARD
           end
         end
         meths.each {|m| send(m) }
-        if @total == 0
+
+        if @undocumented == 0
+          total = 100
+        elsif @total == 0
           total = 0
         else
           total = (@total - @undocumented).to_f / @total.to_f * 100
         end
-        puts("% 3.2f%% documented" % total)
+        log.puts("% 3.2f%% documented" % total)
       end
 
       # Prints list of undocumented objects
       def print_undocumented_objects
         return if !@undoc_list || @undoc_list.empty?
-        puts
-        puts "Undocumented Objects:"
+        log.puts
+        log.puts "Undocumented Objects:"
 
         objects = @undoc_list.sort_by {|o| o.file }
         max = objects.sort_by {|o| o.path.length }.last.path.length
         if @compact
           objects.each do |object|
-            puts("%-#{max}s     (%s)" % [object.path,
+            log.puts("%-#{max}s     (%s)" % [object.path,
               [object.file, object.line].compact.join(":")])
           end
         else
           last_file = nil
           objects.each do |object|
             if object.file != last_file
-              puts
-              puts "(in file: #{object.file})"
+              log.puts
+              log.puts "(in file: #{object.file})"
             end
-            puts object.path
+            log.puts object.path
             last_file = object.file
           end
         end
@@ -125,8 +128,8 @@ module YARD
       # Statistics for methods
       def stats_for_methods
         objs = all_objects.select {|m| m.type == :method }
-        objs.reject! {|m| m.is_alias? || !m.is_explicit? }
-        undoc = objs.select {|m| m.docstring.blank? && !m.overridden_method }
+        objs.reject! {|m| m.is_alias? }
+        undoc = objs.select {|m| m.docstring.blank? }
         @undoc_list |= undoc if @undoc_list
         output "Methods", objs.size, undoc.size
       end
@@ -148,7 +151,7 @@ module YARD
         else
           data = "%5s" % data
         end
-        puts("%-12s %s" % [name + ":", data])
+        log.puts("%-12s %s" % [name + ":", data])
       end
 
       private
