@@ -1,22 +1,18 @@
 require File.join(File.dirname(__FILE__), '..', 'spec_helper')
-begin require 'continuation'; rescue LoadError; end unless RUBY18
 
-class YARD::Parser::CParser; def ensure_loaded!(a, b=1) a end end
-
-describe YARD::Parser::CParser do
+describe YARD::Parser::C::CParser do
   describe '#parse' do
-    before(:all) do
-      file = File.join(File.dirname(__FILE__), 'examples', 'array.c.txt')
-      @parser = Parser::CParser.new(IO.read(file))
-      @parser.parse
-    end
-
-    def parse
+    def parse(contents)
       Registry.clear
-      Parser::CParser.new(@contents).parse
+      YARD.parse_string(contents, :c)
     end
 
     describe 'Array class' do
+      before(:all) do
+        file = File.join(File.dirname(__FILE__), 'examples', 'array.c.txt')
+        parse(File.read(file))
+      end
+
       it "should parse Array class" do
         obj = YARD::Registry.at('Array')
         obj.should_not be_nil
@@ -29,172 +25,116 @@ describe YARD::Parser::CParser do
         obj.tags(:overload).size.should > 1
       end
     end
-    
+
     describe 'Source located in extra files' do
       before(:all) do
         @multifile = File.join(File.dirname(__FILE__), 'examples', 'multifile.c.txt')
         @extrafile = File.join(File.dirname(__FILE__), 'examples', 'extrafile.c.txt')
         @contents = File.read(@multifile)
       end
-      
+
       it "should look for methods in extra files (if 'in' comment is found)" do
         extra_contents = File.read(@extrafile)
         File.should_receive(:read).with('extra.c').and_return(extra_contents)
-        parse
+        parse(@contents)
         Registry.at('Multifile#extra').docstring.should == 'foo'
       end
 
       it "should stop searching for extra source file gracefully if file is not found" do
         File.should_receive(:read).with('extra.c').and_raise(Errno::ENOENT)
         log.should_receive(:warn).with("Missing source file `extra.c' when parsing Multifile#extra")
-        parse
+        parse(@contents)
         Registry.at('Multifile#extra').docstring.should == ''
       end
+
+      it "should differentiate between a struct and a pointer to a struct retval" do
+        parse(@contents)
+        Registry.at('Multifile#hello_mars').docstring.should == 'Hello Mars'
+      end
     end
-    
+
     describe 'Foo class' do
       it 'should not include comments in docstring source' do
-        @contents = <<-eof
-          /* 
+        parse <<-eof
+          /*
            * Hello world
            */
           VALUE foo(VALUE x) {
             int value = x;
           }
-          
+
           void Init_Foo() {
             rb_define_method(rb_cFoo, "foo", foo, 1);
           }
         eof
-        parse
-        Registry.at('Foo#foo').source.gsub(/\s\s+/, ' ').should == 
+        Registry.at('Foo#foo').source.gsub(/\s\s+/, ' ').should ==
           "VALUE foo(VALUE x) { int value = x;\n}"
       end
     end
-    
-    describe 'Defining methods with source in other files' do
-      it "should look in another file for method" do
-        File.should_receive(:read).at_least(1).times.with('file.c').and_return(<<-eof)
-          /* FOO
-           */
-          VALUE foo(VALUE x) 
-          { }
 
-          /* BAR
-           */
-          VALUE bar(VALUE x) 
-          { }
-        eof
-        @contents = <<-eof
-          void Init_Foo() {
-            rb_define_method(rb_cFoo, "foo", foo, 1); /* in file.c */
-            rb_define_global_function("bar", bar, 1); /* in file.c */
+    describe 'Constant' do
+      it 'should not truncate docstring' do
+        parse <<-eof
+          #define MSK_DEADBEEF 0xdeadbeef
+          void
+          Init_Mask(void)
+          {
+              rb_cMask  = rb_define_class("Mask", rb_cObject);
+              /* 0xdeadbeef: This constant is frequently used to indicate a
+               * software crash or deadlock in embedded systems. */
+              rb_define_const(rb_cMask, "DEADBEEF", INT2FIX(MSK_DEADBEEF));
           }
         eof
-        parse
-        Registry.at('Foo#foo').docstring.should == 'FOO'
-        Registry.at('Kernel#bar').docstring.should == 'BAR'
-      end
-
-      it "should allow extra file to include /'s and other filename characters" do
-        File.should_receive(:read).at_least(1).times.with('ext/a-file.c').and_return(<<-eof)
-          /* FOO
-           */
-          VALUE foo(VALUE x) {
-            int value = x;
-          }
-          
-          /* BAR
-           */
-          VALUE bar(VALUE x) {
-            int value = x;
-          }
-        eof
-        @contents = <<-eof
-          void Init_Foo() {
-            rb_define_method(rb_cFoo, "foo", foo, 1); /* in ext/a-file.c */
-            rb_define_global_function("bar", bar, 1); /* in ext/a-file.c */
-          }
-        eof
-        parse
-        Registry.at('Foo#foo').docstring.should == 'FOO'
-        Registry.at('Kernel#bar').docstring.should == 'BAR'
+        constant = Registry.at('Mask::DEADBEEF')
+        constant.value.should == '0xdeadbeef'
+        constant.docstring.should == "This constant is frequently used to indicate a\nsoftware crash or deadlock in embedded systems."
       end
     end
-    
-    describe 'Defining attributes' do
-      before do
-        Registry.clear
-      end
-      
-      def run(read, write, commented = nil)
-        @contents = <<-eof
-          /* FOO */
-          VALUE foo(VALUE x) { int value = x; }
-          void Init_Foo() {
-            rb_cFoo = rb_define_class("Foo", rb_cObject);
-            #{commented ? '/*' : ''} 
-              rb_define_attr(rb_cFoo, "foo", foo, #{read}, #{write});
-            #{commented ? '*/' : ''}
-          }
-        eof
-        parse
-      end
-      
-      it "should handle readonly attribute (rb_define_attr)" do
-        run(1, 0)
-        Registry.at('Foo#foo').should be_reader
-        Registry.at('Foo#foo=').should be_nil
-      end
 
-      it "should handle writeonly attribute (rb_define_attr)" do
-        run(0, 1)
-        Registry.at('Foo#foo').should be_nil
-        Registry.at('Foo#foo=').should be_writer
-      end
+    describe 'Macros' do
+      it "should handle param## inside of macros" do
+        thr = Thread.new do
+          parse <<-eof
+          void
+          Init_gobject_gparamspecs(void)
+          {
+              VALUE cParamSpec = GTYPE2CLASS(G_TYPE_PARAM);
+              VALUE c;
 
-      it "should handle readwrite attribute (rb_define_attr)" do
-        run(1, 1)
-        Registry.at('Foo#foo').should be_reader
-        Registry.at('Foo#foo=').should be_writer
-      end
+          #define DEF_NUMERIC_PSPEC_METHODS(c, typename) \
+            G_STMT_START {\
+              rbg_define_method(c, "initialize", typename##_initialize, 7); \
+              rbg_define_method(c, "minimum", typename##_minimum, 0); \
+              rbg_define_method(c, "maximum", typename##_maximum, 0); \
+              rbg_define_method(c, "range", typename##_range, 0); \
+            } G_STMT_END
 
-      it "should handle commented writeonly attribute (/* rb_define_attr */)" do
-        run(1, 1, true)
-        Registry.at('Foo#foo').should be_reader
-        Registry.at('Foo#foo=').should be_writer
-      end
-    end
-    
-    describe 'Defining aliases' do
-      before do
-        Registry.clear
-      end
-      
-      it "should allow defining of aliases (rb_define_alias)" do
-        @contents = <<-eof
-          /* FOO */
-          VALUE foo(VALUE x) { int value = x; }
-          void Init_Foo() {
-            rb_cFoo = rb_define_class("Foo", rb_cObject);
-            rb_define_method(rb_cFoo, "foo", foo, 1);
-            rb_define_alias(rb_cFoo, "bar", "foo");
-          }
-        eof
-        parse
-        
-        Registry.at('Foo#bar').should be_is_alias
-        Registry.at('Foo#bar').docstring.should == 'FOO'
+          #if 0
+              rbg_define_method(c, "default_value", typename##_default_value, 0); \
+              rb_define_alias(c, "default", "default_value"); \
+
+          #endif
+
+              c = G_DEF_CLASS(G_TYPE_PARAM_CHAR, "Char", cParamSpec);
+              DEF_NUMERIC_PSPEC_METHODS(c, char);
+          eof
+        end
+        thr.join(5)
+        if thr.alive?
+          fail "Did not parse in time"
+          thr.kill
+        end
       end
     end
   end
 
-  describe '#find_override_comment' do
+  describe 'Override comments' do
     before(:all) do
+      Registry.clear
       override_file = File.join(File.dirname(__FILE__), 'examples', 'override.c.txt')
-      @override_parser = Parser::CParser.new(IO.read(override_file)).parse
+      @override_parser = YARD.parse_string(File.read(override_file), :c)
     end
-    
+
     it "should parse GMP::Z class" do
       z = YARD::Registry.at('GMP::Z')
       z.should_not be_nil
@@ -228,4 +168,4 @@ describe YARD::Parser::CParser do
       neg_self.source.should be_nil
     end
   end
-end
\ No newline at end of file
+end
